/*
  cmd_eeprom.c
  EEPROMѥ֥ƥ
 */

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <mon_func.h>
#include "mon_defs.h"
#include "cmd_eeprom.h"

static const char info[]={
    "\r\nEEPROM subsystem ver 0.1\r\n"
    "\r\n"
    "[l] load S-record file to EEPROM.\r\n"
    "[e] expand S-record file to RAM from EEPROM.\r\n"
    "[w] write byte to EEPROM.\r\n"
    "[r] read byte from EEPROM.\r\n"
    "[d] dump contents of EEPROM.\r\n"
    "[a] toggle autoload enable/disable.\r\n"
    "[h] show this message.\r\n"
    "[q] quit.\r\n\r\n"};

void mon_eeprom_mode()
{
    int addr=0;
    char str[PROMPT_BUFFER_SIZE] = "";
    char data;

    if(mon_has_eeprom==0){
        hputstr("eeprom disabled.\r\n");
        return;
    }

    hputstr(info);
    while(1)
    {
        hputstr("eep-sub > ");
	hgetstr(str);
	hputstr("\r\n");
        
        switch(str[0])
        {
        case 'l':
            eep_writesrec();
            break;

        case 'e':
            eep_readsrec();
            break;

        case 'd':
        {
            int i,j;
            char c,ascii[17];
            
            hprintf("dump\r\naddress?(0x%A)",addr);
            hgetstr(str);
            hputstr("\r\n");
            if(str[0]!='\0'){
                addr = strtol(str,NULL,0);
            }
            
            ascii[16]='\0';
            for(i = 0; i < 16; i++) {
                hprintf("0x%A :",addr);
                for( j=0; j<16; j++){
                    c=eep_read_byte(addr+ j);
                    hprintf(" %B",c);
                    ascii[j] = isprint(c) ? c : '.';
                }
                hprintf(" | %s\r\n",ascii);
                addr += 16;
            }
        }
        break;

        case 'w':
            hputstr("byte write\r\naddress:");
            hgetstr(str);
            addr = strtol(str,NULL,0);
            hputstr("\r\n"
                    "data:");
            hgetstr(str);
            hputstr("\r\n");
            data = strtol(str,NULL,0);
            eep_write_byte(addr, data);
            break;

        case 'r':
            hputstr("read\r\naddress:");
            hgetstr(str);
            addr = strtol(str,NULL,0);
            data = eep_read_byte(addr);
            hprintf("\r\n0x%A: %B\r\n", addr, data);
            break;

        case 'a':
        {
            char f;
            f=eep_read_byte(EEP_FLAGS) & EEP_FLAGS_AUTOLOAD;
            hputstr("Now autoload is ");
            hputstr( ( f!= 0) ? "enabled" : "disabled");
            hputstr(".\r\n"
                    "Do you want to ");
            hputstr( ( f!= 0) ? "disable" : "enable");
            hputstr(" autoload?(y/n)");
            hgetstr(str);
            hputstr("\r\n");

            if(str[0]=='y'){
                if(f==0){
                    f=eep_read_byte(EEP_FLAGS) | EEP_FLAGS_AUTOLOAD;
                    eep_write_byte(EEP_FLAGS,f);
                }else{
                    f=eep_read_byte(EEP_FLAGS) & (~EEP_FLAGS_AUTOLOAD);
                    eep_write_byte(EEP_FLAGS,f);
                }
            }else{
                hputstr("configuration unchanged.\r\n");
            }
            hputstr("Now autoload is ");
            hputstr( ( f!= 0) ? "enabled" : "disabled");
            hputstr(".\r\n");
        }
        break;

        case 'h':
            hputstr(info);
            break;

        case 'q':
            return;
        }
    }

}
