/* 
 * Huffman$BK!$K$h$k05=L(B
 *
 * $BJ8;z$N=P8=IQEY$K1~$8$F%3!<%ID9$rJQ2=$5$;$k$3$H$K(B
 * $B$h$C$F05=L$9$k!#(B
 * 
 */

#include <mon_func.h>
#include <limits.h>

#define CN 256			/* $B=P8=$9$k%3!<%I$N<oN`(B */

struct t_node{
    short int p,r,l;
};

/* $B$3$l$i$NJQ?t$O%0%m!<%P%kJQ?t$H$7$?J}$,8+DL$7$,NI$$$,(B
   $B%a%b%j$N@aLs$N$?$a$K!$$3$N9=B$BN$X$N%]%$%s%?$N$_$r@EE*$K3NJ]$9$k(B */

struct global_variables{
    char *dat;			/* $BA`:n$9$k$Y$-%G!<%?(B */
    long int size;		/* $B5v$5$l$k=q$-9~$_NL(B */
    long int c;                 /* $B<!$KA`:n$9$k$Y$-%G!<%?$N0LCV(B */
    short int pnt;
    short int cb;		/* $B<!$KA`:n$9$k$Y$-%S%C%H(B */
    struct t_node node[CN*2-1];
};

static struct global_variables *gv;

static void bitout(int num,int dat)
{
    short int i;
    static int w;

    for( i=0; i<num; i++)
    {
        if(gv->cb==0){
            w=0;
        }else{
            w >>= 1;
        }

        if((dat & 1 )!=0) w |= 0x80;

        gv->cb++;
        if(gv->cb==8){
            gv->dat[gv->c] = w;
            gv->c++;
            gv->cb=0;
            if( gv->c >= gv->size){
                hputstr("huffman.c: outbuf overflow.\r\n");
                return;
            }
        }
        dat >>= 1;
    }
}

inline static int bitin(void)
{
    static int w;
    if(gv->cb==0){
        w=gv->dat[gv->c];
    }else{
        w >>= 1;
    }
    gv->cb++;
    if(gv->cb == 8){
        gv->cb=0;
        gv->c++;
    }
    return (w & 1);
}

inline static int min(int freq[])
{
    int i,min=-1;

    for(i=0;i<CN*2-1;i++){
        if(freq[i]!=0){
            min=i;
            break;
        }
    }

    if(min==-1){
        return -1;
    }

    for(i=0;i<CN*2-1;i++){
        if(freq[i]!=0){ 
            if(freq[i]<freq[min]) min=i;
        }
    }
    return min;
}

static void write_tree(int root)
{
    if(root<CN){
        bitout(1,0);
        bitout(8,root);
/*         fprintf(stderr,"%d\n",root); */
    }else{
        bitout(1,1);
/*         fprintf(stderr,"left>"); */
        write_tree(gv->node[root].l);
/*         fprintf(stderr,"right>"); */
        write_tree(gv->node[root].r);
    }
}

int huffman_encode( unsigned char *in, int isize, char *obuf, int osize)
{
    int freq[CN*2-1],i,m1,m2,printpos=0;
    short int j,k;
    short int c,w,bits;

    struct global_variables s_gv;
    short int avail=CN;
    
    gv=&s_gv;

    gv->dat=obuf;
    gv->size=osize;
    gv->c=0;
    gv->cb=0;

    /* $B%3!<%I$N=P8=IQEY$r%+%&%s%H(B */
    hputstr("huffman.c: pass 1\r\n");
    for( i=0; i<CN*2-1;i++){
        freq[i]=0;
    }

    for( i=0; i<isize;i++){
        freq[(int)in[i]]++;
    }

    for(i=0;i<CN*2-1;i++){
        gv->node[i].p=0;
        gv->node[i].l=0;
        gv->node[i].r=0;
    }

    /* Huffman$BLZ$N9=C[(B */
    hputstr("huffman.c: calculating huffman tree.\r\n");
    while(1)
    {
        j=min(freq);

        /* $B=P8=IQEY:G>.$N#2$D$rA*Br(B */
        m1=freq[j];
        freq[j]=0;
        k=min(freq);
        if(k==-1) break;        /* $B%G!<%?$,0l$D$@$1$J$i=*N;(B */
        m2=freq[k];
        freq[k]=0;

        /* $BFs$D$rB+$M$F0l$D$K(B */
        freq[avail]=m1+m2;

        gv->node[avail].r=j;
        gv->node[avail].l=k;

        gv->node[j].p=avail;
        gv->node[k].p=-avail;

/*         fprintf(stderr,"%d(%c) [%d] : %d(%c) [%d] -> %d [%d]\n" */
/*                 ,j,isprint(j) ? j : '.',m1 */
/*                 ,k,isprint(k) ? k : '.',m2 */
/*                 ,avail,freq[avail]); */

        avail++;
    }

    write_tree(avail-1);

    hputstr("huffman.c: pass 2\r\n");

    for(i=0;i<isize;i++)
    {
        w=0;
        bits=0;

        c=in[i];
/*         fprintf(stderr,"[%d](%c)",c,isprint(c) ? c : '.'); */
        while(gv->node[c].p != 0){
            if(gv->node[c].p<0){
                w <<= 1;
                w |= 1;
                bits++;
                c=-gv->node[c].p;
            }else{
                w <<= 1;
                bits++;
                c=gv->node[c].p;
            }

        }
        bitout(bits,w);
        for(j=0;j<bits;j++){
/*             fprintf(stderr,"%d",w&1); */
            w >>= 1;
        }
        if(i>printpos){
            hprintf("\r%d/%d",i,isize);
            printpos += 1024;
        }
/*         fprintf(stderr,"\n"); */
    }
    hputstr("\r                           \rdone.\r\n");
    bitout(7,0);

/*     out->c=0; */
/*     out->cb=0; */

/*     for(i=0;i<l;i++){ */
/*         fprintf(stderr,"%d",bitin(out)); */
/*     } */

/*      fprintf(stderr,"in:%d byte out:%d byte out/in(%d%%)\n",isize,gv->c, gv->c*100/isize); */
    return gv->c;
}

static int read_tree()
{
    int i,w=0;

    if(bitin()==1){
        i=gv->pnt;
        if(gv->pnt >= 2*CN-1){
            hputstr("huffman.c:table reading error.\r\n");
        }
        gv->pnt++;
/*         fprintf(stderr,"pnt:%d\n",pnt); */
        gv->node[i].l = read_tree();
        gv->node[i].r = read_tree();
        return i;
    }else{
        for(i=0;i<8;i++){
            w |= bitin() << i;
        }
        return w;
    }
}

int huffman_decode( char *in, int size,char *obuf)
{
    int i;
    int j,root;
    struct global_variables s_gv;

    gv = &s_gv;

    gv->dat=in;
    gv->size=INT_MAX;
    gv->c=0;
    gv->cb=0;

    gv->pnt=CN;
    root=read_tree(CN);

/*     fprintf(stderr,"size:%d root:%d\n",size,root); */
    for(i=0;i<size;i++)
    {
        j=root;
        while(j>=CN){
            if(bitin()!=0){
                j=gv->node[j].l;
            }else{
                j=gv->node[j].r;
            }
        }
/*         fprintf(stderr,"%c",j); */
        obuf[i]=j;
    }
    return 0;
}


